/*
 * orbit.c
 */
#include "copyright.h"

#include <stdio.h>
#include <math.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include "defs.h"
#include "struct.h"
#include "data.h"
#include "packets.h"
#ifndef _NO_DEPEND
#   include "ntserv_protos.h"
#endif

#ifdef USE_SYS_PROTOS
#if defined(__STDC__) || defined(__cplusplus)
#define P_(s) s
#else
#define P_(s) ()
#endif

extern void                   perror P_((const char *));

#undef P_
#endif

/* orbit a planet */

void
orbit()
{
   register int    i;
   register struct planet *l;
   unsigned char   dir;
   int             dx, dy;

   if (me->p_flags & PFORBIT)	/* bug fix: no reverse/rapid oribiting
				 * 6/23/92 TMC */
      return;

   if (me->p_speed > ORBSPEED) {
      warning(0, "Helmsman: Captain, the maximum safe speed for docking or orbiting is warp 2!", 79);
      return;
   }
   me->p_flags &= ~(PFORBIT | PFPLOCK | PFPLLOCK | PFTRACT | PFPRESS);

   if (me->p_flags & PFDOCK) {
#ifdef SB_DOCK
      players[me->p_docked].p_numdocked--;
      players[me->p_docked].p_port[me->p_portdocked] = VACANT;
#else
      players[me->p_docked].p_docked--;
      players[me->p_docked].p_port[me->p_port[0]] = VACANT;
#endif
      me->p_flags &= ~PFDOCK;
   }
   if (me->p_ship.s_type != STARBASE
#ifdef SB_DOCK
       || sb_dock
#endif
      ) {
      for (i = 0; i < MAXPLAYER; i++) {
	 if (me->p_no == i)
	    continue;
	 if (players[i].p_ship.s_type != STARBASE)
	    continue;
	 if (!isAlive(&players[i]))
	    continue;
	 if (!friendlyPlayer(&players[i]))
	    continue;
	 dx = ABS(players[i].p_x - me->p_x);
	 dy = ABS(players[i].p_y - me->p_y);
	 if (dx > DOCKDIST || dy > DOCKDIST)
	    continue;

	 dock(i);
	 return;
      }
   }
   for (i = 0, l = &planets[i]; i < MAXPLANETS; i++, l++) {
      dx = ABS(l->pl_x - me->p_x);
      dy = ABS(l->pl_y - me->p_y);
      if (dx > ENTORBDIST || dy > ENTORBDIST)	/* XXX */
	 continue;
      if (dx * dx + dy * dy > ENTORBDIST * ENTORBDIST)
	 continue;
      if (!(me->p_team & planets[i].pl_owner) &&
	  me->p_ship.s_type == STARBASE) {
	 warning(1, "Central Command regulations prohibits you from orbiting foreign planets", 80);
	 return;
      }
      if(me->p_x == l->pl_x && l->pl_y == me->p_y)
	 dir = 0;
      else
	 dir = (unsigned char) (atan2((double) (me->p_x - l->pl_x),
				      (double) (l->pl_y - me->p_y))
				/ 3.14159 * 128.);
      MATH_ERR_CHECK(me->p_x-l->pl_x + l->pl_y-me->p_y);
      l->pl_info |= me->p_team;
      me->p_dir = dir + 64;
      me->p_flags |= PFORBIT;
      me->p_x = l->pl_x + ORBDIST * Cos[dir];
      me->p_y = l->pl_y + ORBDIST * Sin[dir];
      me->p_speed = me->p_desspeed = 0;
      me->p_planet = l->pl_no;
      return;
   }

   warning(1, "Helmsman:  Sensors read no valid targets in range to dock or orbit sir!", 81);
}


void
dock(base_id)
   int             base_id;
{
   int             port_id = NUMPORTS;
   char            buf[80];

   if (!(players[base_id].p_flags & PFDOCKOK)) {
#ifdef SHORT_PACKETS
      if(send_short){
	 swarning(SBDOCKREFUSE_TEXT, base_id, 0);
      }
      else 
#endif
      {
	 sprintf(buf, "Starbase %s refusing us docking permission captain.",
		 players[base_id].p_name);
	 warning(1, buf, -1);
      }
#ifdef SB_TRANSWARP
      if (me->p_flags & PFTWARP) {
	 me->p_flags &= ~PFTWARP;
	 set_speed(0);
      }
#endif
      return;
   }
   /* Try to dock on closest port, else pick first vacant port. */
   if ((me->p_x < players[base_id].p_x && me->p_y < players[base_id].p_y) && (players[base_id].p_port[3] == VACANT)
#ifdef SB_DOCK
       && !((players[base_id].p_flags & PFDOCK) && players[base_id].p_portdocked == 3)
#endif
      ) {
      port_id = 3;
   } else if ((me->p_x < players[base_id].p_x && me->p_y > players[base_id].p_y) && (players[base_id].p_port[2] == VACANT)
#ifdef SB_DOCK
	      && !((players[base_id].p_flags & PFDOCK) && players[base_id].p_portdocked == 2)
#endif
      ) {
      port_id = 2;
   } else if ((me->p_x > players[base_id].p_x && me->p_y > players[base_id].p_y) && (players[base_id].p_port[1] == VACANT)
#ifdef SB_DOCK
	      && !((players[base_id].p_flags & PFDOCK) && players[base_id].p_portdocked == 1)
#endif
      ) {
      port_id = 1;
   } else if ((me->p_x > players[base_id].p_x && me->p_y < players[base_id].p_y) && (players[base_id].p_port[0] == VACANT)
#ifdef SB_DOCK
	      && !((players[base_id].p_flags & PFDOCK) && players[base_id].p_portdocked == 0)
#endif
      ) {
      port_id = 0;
   } else if (players[base_id].p_port[0] == VACANT
#ifdef SB_DOCK
	      && !((players[base_id].p_flags & PFDOCK) && players[base_id].p_portdocked == 0)
#endif
      ) {
      port_id = 0;
   } else if (players[base_id].p_port[1] == VACANT
#ifdef SB_DOCK
	      && !((players[base_id].p_flags & PFDOCK) && players[base_id].p_portdocked == 1)
#endif
      ) {
      port_id = 1;
   } else if (players[base_id].p_port[2] == VACANT
#ifdef SB_DOCK
	      && !((players[base_id].p_flags & PFDOCK) && players[base_id].p_portdocked == 2)
#endif
      ) {
      port_id = 2;
   } else if (players[base_id].p_port[3] == VACANT
#ifdef SB_DOCK
	      && !((players[base_id].p_flags & PFDOCK) && players[base_id].p_portdocked == 3)
#endif
      ) {
      port_id = 3;
   }
   if (port_id < NUMPORTS) {
      me->p_x = players[base_id].p_x + DOCKDIST * Cos[(port_id * 90 + 45) * 255 / 360];
      me->p_y = players[base_id].p_y + DOCKDIST * Sin[(port_id * 90 + 45) * 255 / 360];
      me->p_speed = 0;
      me->p_desspeed = 0;
      me->p_dir = (90 + port_id * 90 + 45) * 255 / 360;
      me->p_desdir = me->p_dir;

      me->p_flags |= PFDOCK;
#ifdef SB_DOCK
      players[base_id].p_numdocked++;
#else
      players[base_id].p_docked++;
#endif
      players[base_id].p_port[port_id] = me->p_no;
      me->p_docked = base_id;
#ifdef SB_DOCK
      me->p_portdocked = port_id;
#else
      me->p_port[0] = port_id;
#endif
#ifdef SHORT_PACKETS
      if(send_short){
	 swarning(ONEARG_TEXT, 2, port_id);
      }
      else
#endif
      {
	 sprintf(buf, "Helmsman:  Docking manuever completed Captain.  All moorings secured at port %d.", port_id);
	 warning(1, buf, -1);
      }
#ifdef SB_TRANSWARP
      if (me->p_flags & PFTWARP){
	 me->p_flags &= ~PFTWARP;
	 me->p_flags |= PFREFITTING;
	 rdelay = me->p_updates + 50;
      }
#endif
   } else {
#ifdef SHORT_PACKETS
      if(send_short){
	 swarning(SBDOCKDENIED_TEXT, base_id, 0);
	 return;
      }
#endif
      sprintf(buf, "Starbase %s: Permission to dock denied, all ports currently occupied.", players[base_id].p_name);
      warning(1, buf, -1);
   }
}				/* dock() */
